@@ -13,7 +13,7 @@
 #
 
 import os, sys, re, shutil, glob
-from SCons.Util import WhereIs
+from SCons.Util import *
 
 
 def getVerFromConfigure(path):
@@ -33,6 +33,27 @@ def getVerFromConfigure(path):
     return 'x.x.x'
 
 
+def relativePath(path, base):
+    '''return relative path from base, which is usually top source dir'''
+    # full pathname of path
+    path1 = os.path.normpath(os.path.realpath(path)).split(os.sep)
+    path2 = os.path.normpath(os.path.realpath(base)).split(os.sep)
+    if path1[:len(path2)] != path2:
+        print "Path %s is not under top source directory" % path
+    if len(path2) == len(path1):
+        return ''
+    path3 = os.path.join(*path1[len(path2):]);
+    # replace all \ by / such that we get the same comments on Windows and *nix
+    path3 = path3.replace('\\', '/')
+    return path3
+
+
+def isSubDir(path, base):
+    '''Whether or not path is a subdirectory of base'''
+    path1 = os.path.normpath(os.path.realpath(path)).split(os.sep)
+    path2 = os.path.normpath(os.path.realpath(base)).split(os.sep)
+    return len(path2) <= len(path1) and path1[:len(path2)] == path2
+
 
 def writeToFile(filename, lines, append = False):
     " utility function: write or append lines to filename "
@@ -68,7 +89,6 @@ def env_subst(target, source, env):
             # multi-line replacement
             val = val.replace('\n',r'\\n\\\n')
             contents = re.sub('@'+k+'@', val, contents)
-            contents = re.sub('%'+k+'%', val, contents)
         except:
             pass
     target_file.write(contents + "\n")
@@ -77,6 +97,70 @@ def env_subst(target, source, env):
     #os.chmod(str(target[0]), stat.S_IMODE(st[stat.ST_MODE]) | stat.S_IWRITE)
 
 
+def env_nsis(source, target, env, for_signature):
+    ''' Get nsis command line '''
+    def quoteIfSpaced(str):
+        if ' ' in str:
+            return '"' + str + '"'
+        else:
+            return str
+    ret = env['NSIS'] + " /V1 "
+    if env.has_key('NSISFLAGS'):
+        for flag in env['NSISFLAGS']:
+            ret += flag
+            ret += ' '
+    if env.has_key('NSISDEFINES'):
+        for d in env['NSISDEFINES']:
+            ret += '/D'+d
+            if env['NSISDEFINES'][d]:
+                ret += '=' + quoteIfSpaced(env['NSISDEFINES'][d])
+            ret += ' '
+    # bundled?
+    if '-bundle.exe' in str(target[0]):
+        ret += '/DSETUPTYPE_BUNDLE=1 '
+    for s in source:
+        ret += quoteIfSpaced(str(s))
+    return ret
+
+
+def env_toc(target, source, env):
+    '''Generate target from source files'''
+    # this is very tricky because we need to use installed lyx2lyx with 
+    # correct lyx2lyx_version.py
+    sys.path.append(env['LYX2LYX_DEST'])
+    sys.path.append(env.Dir('$TOP_SRCDIR/lib/doc').abspath)
+    import doc_toc
+    # build toc
+    doc_toc.build_toc(str(target[0]), [file.abspath for file in source])
+    
+    
+def env_cat(target, source, env):
+    '''Cat source > target. Avoid pipe to increase portability'''
+    output = open(env.File(target[0]).abspath, 'w')
+    for src in source:
+        input = open(env.File(src).abspath)
+        output.write(input.read())
+        input.close()
+    output.close()
+
+
+def env_potfiles(target, source, env):
+    '''Build po/POTFILES.in'''
+    # command 
+    #   grep -l '_(\".*\")' `find src \( -name '*.h' -o -name '*.cpp' -o -name '*.cpp.in' \) -print` | grep -v -e "src/support/Package.cpp$$" | sort | uniq
+    # is used under *nix but windows users have to do these all in python
+    target_file = open(str(target[0]), "w")
+    potfiles = []
+    trans = re.compile('_\(".*"\)', re.M)
+    for file in source:
+        rel_file = relativePath(str(file), env.subst('$TOP_SRCDIR'))
+        if rel_file not in potfiles and trans.search(open(str(file)).read()):
+            potfiles.append(rel_file)
+    potfiles.sort()
+    print >> target_file, '\n'.join(potfiles)
+    target_file.close()
+
+    
 def createResFromIcon(env, icon_file, rc_file):
     ''' create a rc file with icon, and return res file (windows only) '''
     if os.name == 'nt':
@@ -134,7 +218,7 @@ int main()
 
 
 def checkCXXGlobalCstd(conf):
-    ''' Check the use of std::tolower or tolower '''
+    ''' Checking the use of std::tolower or tolower '''
     check_global_cstd_source = '''
 #include <cctype>
 using std::tolower;
@@ -143,8 +227,8 @@ int main()
     return 0;
 }
 '''
-    conf.Message('Check for the use of global cstd... ')
-    ret = conf.TryLink(check_global_cstd_source, '.c')
+    conf.Message('Checking for the use of global cstd... ')
+    ret = conf.TryLink(check_global_cstd_source, '.cpp')
     conf.Result(ret)
     return ret
 
@@ -272,6 +356,30 @@ def checkCommand(conf, cmd):
     return res
 
 
+def checkNSIS(conf):
+    ''' check the existence of nsis compiler, return the fullpath '''
+    conf.Message('Checking for nsis compiler...')
+    res = None
+    if can_read_reg:
+        # If we can read the registry, get the NSIS command from it
+        try:
+            k = RegOpenKeyEx(hkey_mod.HKEY_LOCAL_MACHINE,
+                                  'SOFTWARE\\NSIS')
+            val, tok = RegQueryValueEx(k,None)
+            ret = val + os.path.sep + 'makensis.exe'
+            if os.path.isfile(ret):
+                res = '"' + ret + '"'
+            else:
+                res = None
+        except:
+            pass # Couldn't find the key, just act like we can't read the registry
+    # Hope it's on the path
+    if res is None:
+        res = WhereIs('makensis.exe')
+    conf.Result(res is not None)
+    return res
+
+
 def checkLC_MESSAGES(conf):
     ''' check the definition of LC_MESSAGES '''
     check_LC_MESSAGES = '''
@@ -281,7 +389,7 @@ int main()
     return LC_MESSAGES;
 }
 '''
-    conf.Message('Check for LC_MESSAGES in locale.h... ')
+    conf.Message('Checking for LC_MESSAGES in locale.h... ')
     ret = conf.TryLink(check_LC_MESSAGES, '.c')
     conf.Result(ret)
     return ret
@@ -298,15 +406,52 @@ int main() {
     return 0; 
 }
 '''
-    conf.Message('Check if the declaration of iconv needs const... ')
-    ret = conf.TryLink(check_iconv_const, '.c')
+    conf.Message('Checking if the declaration of iconv needs const... ')
+    ret = conf.TryLink(check_iconv_const, '.cpp')
     conf.Result(ret)
     return ret
 
 
+def checkSizeOfWChar(conf):
+    ''' check the size of wchar '''
+    check_sizeof_wchar = '''
+int i[ ( sizeof(wchar_t)==%d ? 1 : -1 ) ];
+int main()
+{
+    return 0;
+}
+'''
+    conf.Message('Checking the size of wchar_t... ')
+    if conf.TryLink(check_sizeof_wchar % 2, '.cpp'):
+        ret = 2
+    elif conf.TryLink(check_sizeof_wchar % 4, '.cpp'):
+        ret = 4
+    else:
+        ret = 0
+    conf.Result(str(ret))
+    return ret
+
+
+def checkDeclaration(conf, func, headers):
+    ''' check if a function is declared in given headers '''
+    check_decl = '''
+#include <%%s>
+int main()
+{
+#ifndef %s
+    char *p = (char *) %s;
+#endif
+}
+''' % (func, func)
+    conf.Message('Checking for the declaration of function %s... ' % func)
+    ret = True in [conf.TryLink(check_decl % header, '.c') for header in headers]
+    conf.Result(ret)
+    return ret
+
+    
 def createConfigFile(conf, config_file,
     config_pre = '', config_post = '',
-    headers = [], functions = [], types = [], libs = [],
+    headers = [], functions = [], declarations = [], types = [], libs = [],
     custom_tests = [], extra_items = []):
     ''' create a configuration file, with options
         config_file: which file to create
@@ -316,6 +461,8 @@ def createConfigFile(conf, config_file,
             ('file', 'HAVE_FILE', 'c'/'c++')
         functions: functions to check, in the form of a list of
             ('func', 'HAVE_func', 'include lines'/None)
+        declarations: function declarations to check, in the form of a list of
+            ('func', 'HAVE_DECL_func', header_files)
         types: types to check, in the form of a list of
             ('type', 'HAVE_TYPE', 'includelines'/None)
         libs: libraries to check, in the form of a list of
@@ -364,6 +511,14 @@ def createConfigFile(conf, config_file,
         else:
             result[func[1]] = 0
             cont += configString('/* #undef %s */' % func[1], desc = description)
+    for decl in declarations:
+        description = "Define to 1 if you have the declaration of `%s', and to 0 if you don't." % decl[0]
+        if conf.CheckDeclaration(decl[0], decl[2]):
+            result[decl[1]] = 1
+            cont += configString('#define %s 1' % decl[1], desc = description)
+        else:
+            result[decl[1]] = 0
+            cont += configString('/* #undef %s */' % decl[1], desc = description)
     # types
     for t in types:
         description = "Define to 1 if you have the `%s' type." % t[0]
