@@ -1,4 +1,5 @@
 #! /usr/bin/env python
+# -*- coding: utf-8 -*-
 #
 # file configure.py
 # This file is part of LyX, the document processor.
@@ -7,26 +8,8 @@
 # \author Bo Peng
 # Full author contact details are available in file CREDITS.
 
-# This is an experimental version of the configure script, written
-# in Python. 
-
 import sys, os, re, shutil, glob
 
-# compatibility with python 2.2
-if sys.version_info[:2] == (2, 2):
-    __builtins__.True = (1 == 1)
-    __builtins__.False = (1 == 0)
-    def bool(value):
-        """Demote a value to 0 or 1, depending on its truth value
-
-        This is not to be confused with types.BooleanType, which is
-        way too hard to duplicate in 2.1 to be worth the trouble.
-        """
-        return not not value
-    __builtins__.bool = bool
-    del bool
-# end compatibility chunk
-
 
 class Tee:
     ''' Writing to a Tee object will write to all file objects it keeps.
@@ -114,20 +97,25 @@ def createDirectories():
 def checkTeXPaths():
     ''' Determine the path-style needed by the TeX engine on Win32 (Cygwin) '''
     windows_style_tex_paths = ''
-    if sys.platform == 'cygwin':
+    if os.name == 'nt' or sys.platform == 'cygwin':
         from tempfile import mkstemp
-        fd, tmpfname = mkstemp(suffix='.ltx', dir='/tmp')
+        fd, tmpfname = mkstemp(suffix='.ltx')
+        if os.name == 'nt':
+            inpname = tmpfname.replace('\\', '/')
+        else:
+            inpname = cmdOutput('cygpath -m ' + tmpfname)
+        logname = os.path.basename(inpname.replace('.ltx', '.log'))
+        inpname = inpname.replace('~', '\\string~')
         os.write(fd, r'\relax')
         os.close(fd)
-        inpname = cmdOutput('cygpath -m ' + tmpfname)
         latex_out = cmdOutput(r'latex "\nonstopmode\input{%s}"' % inpname)
-        if latex_out.find('Error') != -1:
+        if 'Error' in latex_out:
             print "configure: TeX engine needs posix-style paths in latex files"
             windows_style_tex_paths = 'false'
         else:
             print "configure: TeX engine needs windows-style paths in latex files"
             windows_style_tex_paths = 'true'
-        removeFiles([tmpfname, 'texput.log'])
+        removeFiles([tmpfname, logname, 'texput.log'])
     return windows_style_tex_paths
 
 
@@ -170,17 +158,19 @@ def checkProg(description, progs, rc_entry = [], path = [], not_found = ''):
         ac_word = ac_prog.split(' ')[0]
         print '+checking for "' + ac_word + '"... ',
         path = os.environ["PATH"].split(os.pathsep) + path
+        extlist = ['']
+        if os.environ.has_key("PATHEXT"):
+            extlist = extlist + os.environ["PATHEXT"].split(os.pathsep)
         for ac_dir in path:
-            # check both ac_word and ac_word.exe (for windows system)
-            if os.path.isfile( os.path.join(ac_dir, ac_word) ) or \
-                os.path.isfile( os.path.join(ac_dir, ac_word + ".exe") ):
-                print ' yes'
-                # write rc entries for this command
-                if len(rc_entry) == 1:
-                    addToRC(rc_entry[0].replace('%%', ac_prog))
-                elif len(rc_entry) > 1:
-                    addToRC(rc_entry[idx].replace('%%', ac_prog))
-                return [ac_dir, ac_word]
+            for ext in extlist:
+                if os.path.isfile( os.path.join(ac_dir, ac_word + ext) ):
+                    print ' yes'
+                    # write rc entries for this command
+                    if len(rc_entry) == 1:
+                        addToRC(rc_entry[0].replace('%%', ac_prog))
+                    elif len(rc_entry) > 1:
+                        addToRC(rc_entry[idx].replace('%%', ac_prog))
+                    return [ac_dir, ac_word]
         # if not successful
         print ' no'
     # write rc entries for 'not found'
@@ -208,21 +198,24 @@ def checkDTLtools():
 
 def checkLatex(dtl_tools):
     ''' Check latex, return lyx_check_config '''
+    path, LATEX = checkProg('a Latex2e program', ['latex $$i', 'latex2e $$i'])
+    path, PPLATEX = checkProg('a DVI postprocessing program', ['pplatex $$i'])
+    # use LATEX to convert from latex to dvi if PPLATEX is not available    
+    if PPLATEX == '':
+        PPLATEX = LATEX
     if dtl_tools:
         # Windows only: DraftDVI
-        converter_entry = r'''\converter latex      dvi2       "%%"	"latex"
-\converter dvi2       dvi        "python -tt $$s/scripts/clean_dvi.py $$i $$o"	""'''
+        addToRC(r'''\converter latex      dvi2       "%s"	"latex"
+\converter dvi2       dvi        "python -tt $$s/scripts/clean_dvi.py $$i $$o"	""''' % PPLATEX)
     else:
-        converter_entry = r'\converter latex      dvi        "%%"	"latex"'
-    path, LATEX = checkProg('a Latex2e program', ['pplatex $$i', 'latex $$i', 'latex2e $$i'],
-        rc_entry = [converter_entry])
+        addToRC(r'\converter latex      dvi        "%s"	"latex"' % PPLATEX)
     # no latex
     if LATEX != '':
         # Check if latex is usable
         writeToFile('chklatex.ltx', '''
 \\nonstopmode\\makeatletter
 \\ifx\\undefined\\documentclass\\else
-    \\message{ThisIsLaTeX2e}
+  \\message{ThisIsLaTeX2e}
 \\fi
 \\@@end
 ''')
@@ -249,74 +242,87 @@ def checkFormatEntries(dtl_tools):
         rc_entry = [r'\Format agr        agr     Grace                  "" "%%"	"%%"	"vector"'])
     #
     checkViewer('a FEN viewer and editor', ['xboard -lpf $$i -mode EditPosition'],
-        rc_entry = [r'\Format fen        fen     FEN                    "" "%%"	"%%"'])
+        rc_entry = [r'\Format fen        fen     FEN                    "" "%%"	"%%"	""'])
     #
-    path, iv = checkViewer('a raster image viewer', ['xv', 'kview', 'gimp'])
-    path, ie = checkViewer('a raster image editor', ['gimp'])
-    addToRC(r'''\Format bmp        bmp     BMP                    "" "%s"	"%s"
-\Format gif        gif     GIF                    "" "%s"	"%s"
-\Format jpg        jpg     JPEG                   "" "%s"	"%s"
-\Format pbm        pbm     PBM                    "" "%s"	"%s"
-\Format pgm        pgm     PGM                    "" "%s"	"%s"
-\Format png        png     PNG                    "" "%s"	"%s"
-\Format ppm        ppm     PPM                    "" "%s"	"%s"
-\Format tiff       tif     TIFF                   "" "%s"	"%s"
-\Format xbm        xbm     XBM                    "" "%s"	"%s"
-\Format xpm        xpm     XPM                    "" "%s"	"%s"''' % \
+    path, iv = checkViewer('a raster image viewer', ['xv', 'kview', 'gimp-remote', 'gimp'])
+    path, ie = checkViewer('a raster image editor', ['gimp-remote', 'gimp'])
+    addToRC(r'''\Format bmp        bmp     BMP                    "" "%s"	"%s"	""
+\Format gif        gif     GIF                    "" "%s"	"%s"	""
+\Format jpg        jpg     JPEG                   "" "%s"	"%s"	""
+\Format pbm        pbm     PBM                    "" "%s"	"%s"	""
+\Format pgm        pgm     PGM                    "" "%s"	"%s"	""
+\Format png        png     PNG                    "" "%s"	"%s"	""
+\Format ppm        ppm     PPM                    "" "%s"	"%s"	""
+\Format tiff       tif     TIFF                   "" "%s"	"%s"	""
+\Format xbm        xbm     XBM                    "" "%s"	"%s"	""
+\Format xpm        xpm     XPM                    "" "%s"	"%s"	""''' % \
         (iv, ie, iv, ie, iv, ie, iv, ie, iv, ie, iv, ie, iv, ie, iv, ie, iv, ie, iv, ie) )
     #
-    checkViewer('a text editor', ['xemacs', 'gvim', 'kedit', 'kwrite', 'kate', \
+    checkViewer('a text editor', ['sensible-editor', 'xemacs', 'gvim', 'kedit', 'kwrite', 'kate', \
         'nedit', 'gedit', 'notepad'],
-        rc_entry = [r'''\Format asciichess asc    "Plain text (chess output)"  "" ""	"%%"
-\Format asciiimage asc    "Plain text (image)"         "" ""	"%%"
-\Format asciixfig  asc    "Plain text (Xfig output)"   "" ""	"%%"
-\Format dateout    tmp    "date (output)"         "" ""	"%%"
-\Format docbook    sgml    DocBook                B  ""	"%%"
-\Format docbook-xml xml   "Docbook (XML)"         "" ""	"%%"
-\Format lilypond   ly     "LilyPond music"        "" ""	"%%"
-\Format literate   nw      NoWeb                  N  ""	"%%"
-\Format latex      tex    "LaTeX (plain)"         L  ""	"%%"
-\Format linuxdoc   sgml    LinuxDoc               x  ""	"%%"
-\Format pdflatex   tex    "LaTeX (pdflatex)"      "" ""	"%%"
-\Format text       txt    "Plain text"            a  ""	"%%"
-\Format textparagraph txt "Plain text (paragraphs)"    "" ""	"%%"''' ])
+        rc_entry = [r'''\Format asciichess asc    "Plain text (chess output)"  "" ""	"%%"	""
+\Format asciiimage asc    "Plain text (image)"         "" ""	"%%"	""
+\Format asciixfig  asc    "Plain text (Xfig output)"   "" ""	"%%"	""
+\Format dateout    tmp    "date (output)"         "" ""	"%%"	""
+\Format docbook    sgml    DocBook                B  ""	"%%"	"document"
+\Format docbook-xml xml   "Docbook (XML)"         "" ""	"%%"	"document"
+\Format literate   nw      NoWeb                  N  ""	"%%"	"document"
+\Format lilypond   ly     "LilyPond music"        "" ""	"%%"	"vector"
+\Format latex      tex    "LaTeX (plain)"         L  ""	"%%"	"document"
+\Format linuxdoc   sgml    LinuxDoc               x  ""	"%%"	"document"
+\Format pdflatex   tex    "LaTeX (pdflatex)"      "" ""	"%%"	"document"
+\Format text       txt    "Plain text"            a  ""	"%%"	"document"
+\Format text2      txt    "Plain text (pstotext)" "" ""	"%%"	"document"
+\Format text3      txt    "Plain text (ps2ascii)" "" ""	"%%"	"document"
+\Format text4      txt    "Plain text (catdvi)"   "" ""	"%%"	"document"
+\Format textparagraph txt "Plain Text, Join Lines" "" ""	"%%"	"document"''' ])
     #
     #checkProg('a Postscript interpreter', ['gs'],
     #  rc_entry = [ r'\ps_command "%%"' ])
-    checkViewer('a Postscript previewer', ['gv', 'ghostview -swap', 'kghostview'],
+    checkViewer('a Postscript previewer', ['kghostview', 'evince', 'gv', 'ghostview -swap'],
         rc_entry = [r'''\Format eps        eps     EPS                    "" "%%"	""	"vector"
-\Format ps         ps      Postscript             t  "%%"	""	"vector"'''])
+\Format ps         ps      Postscript             t  "%%"	""	"document,vector"'''])
     #
-    checkViewer('a PDF previewer', ['acrobat', 'acroread', 'gv', 'ghostview', \
-                            'xpdf', 'kpdf', 'kghostview'],
-        rc_entry = [r'''\Format pdf        pdf    "PDF (ps2pdf)"          P  "%%"	""	"vector"
-\Format pdf2       pdf    "PDF (pdflatex)"        F  "%%"	""	"vector"
-\Format pdf3       pdf    "PDF (dvipdfm)"         m  "%%"	""	"vector"'''])
+    checkViewer('a PDF previewer', ['kpdf', 'evince', 'kghostview', 'xpdf', 'acrobat', 'acroread', \
+		    'gv', 'ghostview'],
+        rc_entry = [r'''\Format pdf        pdf    "PDF (ps2pdf)"          P  "%%"	""	"document,vector"
+\Format pdf2       pdf    "PDF (pdflatex)"        F  "%%"	""	"document,vector"
+\Format pdf3       pdf    "PDF (dvipdfm)"         m  "%%"	""	"document,vector"'''])
     #
     checkViewer('a DVI previewer', ['xdvi', 'kdvi'],
-        rc_entry = [r'\Format dvi        dvi     DVI                    D  "%%"	""	"vector"'])
+        rc_entry = [r'\Format dvi        dvi     DVI                    D  "%%"	""	"document,vector"'])
     if dtl_tools:
         # Windows only: DraftDVI
-        addToRC(r'\Format dvi2       dvi     DraftDVI               ""	""	"vector"')
+        addToRC(r'\Format dvi2       dvi     DraftDVI               ""	""	""	"vector"')
     #
-    checkViewer('a HTML previewer', ['mozilla file://$$p$$i', 'netscape'],
-        rc_entry = [r'\Format html       html    HTML                   H  "%%"	""'])
+    checkViewer('an HTML previewer', ['firefox', 'mozilla file://$$p$$i', 'netscape'],
+        rc_entry = [r'\Format html       html    HTML                   H  "%%"	""	"document"'])
+    #
+    checkViewer('Noteedit', ['noteedit'],
+        rc_entry = [r'\Format noteedit   not     Noteedit               "" "%%"	"%%"	"vector"'])
+    #
+    checkViewer('an OpenDocument viewer', ['oowriter'],
+        rc_entry = [r'\Format odt        odt     OpenDocument           "" "%%"	"%%"	"document,vector"'])
     #
     # entried that do not need checkProg
     addToRC(r'''\Format date       ""     "date command"          "" ""	""	""
-\Format fax        ""      Fax                    "" ""	""	""
+\Format fax        ""      Fax                    "" ""	""	"document"
 \Format lyx        lyx     LyX                    "" ""	""	""
-\Format lyx13x     lyx13  "LyX 1.3.x"             "" ""	""	""
+\Format lyx13x     lyx13  "LyX 1.3.x"             "" ""	""	"document"
+\Format lyx14x     lyx14  "LyX 1.4.x"             "" ""	""	"document"
+\Format clyx       cjklyx "CJK LyX 1.4.x (big5)"  "" ""	""	"document"
+\Format jlyx       cjklyx "CJK LyX 1.4.x (euc-jp)" "" ""	""	"document"
+\Format klyx       cjklyx "CJK LyX 1.4.x (euc-kr)" "" ""	""	"document"
 \Format lyxpreview lyxpreview "LyX Preview"       "" ""	""	""
 \Format pdftex     pdftex_t PDFTEX                "" ""	""	""
 \Format program    ""      Program                "" ""	""	""
 \Format pstex      pstex_t PSTEX                  "" ""	""	""
-\Format rtf        rtf    "Rich Text Format"      "" ""	""	"vector"
-\Format sxw        sxw    "OpenOffice.Org Writer" O  ""	""	"vector"
+\Format rtf        rtf    "Rich Text Format"      "" ""	""	"document,vector"
+\Format sxw        sxw    "OpenOffice.Org (sxw)"  ""  ""	""	"document,vector"
 \Format wmf        wmf    "Windows Metafile"      "" ""	""	"vector"
 \Format emf        emf    "Enhanced Metafile"     "" ""	""	"vector"
-\Format word       doc    "MS Word"               W  ""	""	"vector"
-\Format wordhtml   html   "MS Word (HTML)"        "" ""        ""	""
+\Format word       doc    "MS Word"               W  ""	""	"document,vector"
+\Format wordhtml   html   "HTML (MS Word)"        "" ""        ""	"document"
 ''')
 
 
@@ -344,20 +350,40 @@ def checkConverterEntries():
     checkProg('a Noweb -> LaTeX converter', ['noweave -delay -index $$i > $$o'],
         rc_entry = [ r'\converter literate   latex      "%%"	""' ])
     #
-    checkProg('a HTML -> LaTeX converter', ['html2latex $$i'],
+    checkProg('an HTML -> LaTeX converter', ['html2latex $$i'],
         rc_entry = [ r'\converter html       latex      "%%"	""' ])
     #
-    checkProg('a MSWord -> LaTeX converter', ['wvCleanLatex $$i $$o'],
+    checkProg('an MS Word -> LaTeX converter', ['wvCleanLatex $$i $$o'],
         rc_entry = [ r'\converter word       latex      "%%"	""' ])
-    #
-    checkProg('a LaTeX -> MS Word converter', ["htlatex $$i 'html,word' 'symbol/!' '-cvalidate'"],
-        rc_entry = [ r'\converter latex      wordhtml   "%%"	""' ])
+    # On SuSE the scripts have a .sh suffix, and on debian they are in /usr/share/tex4ht/
+    path, htmlconv = checkProg('a LaTeX -> HTML converter', ['htlatex $$i', 'htlatex.sh $$i', \
+        '/usr/share/tex4ht/htlatex $$i', 'tth  -t -e2 -L$$b < $$i > $$o', \
+        'latex2html -no_subdir -split 0 -show_section_numbers $$i', 'hevea -s $$i'],
+        rc_entry = [ r'\converter latex      html       "%%"	"needaux"' ])
+    if htmlconv.find('htlatex') >= 0 or htmlconv == 'latex2html':
+      addToRC(r'''\copier    html       "python -tt $$s/scripts/ext_copy.py -e html,png,css $$i $$o"''')
+    else:
+      addToRC(r'''\copier    html       "python -tt $$s/scripts/ext_copy.py $$i $$o"''')
+
+    # On SuSE the scripts have a .sh suffix, and on debian they are in /usr/share/tex4ht/
+    path, htmlconv = checkProg('a LaTeX -> MS Word converter', ["htlatex $$i 'html,word' 'symbol/!' '-cvalidate'", \
+        "htlatex.sh $$i 'html,word' 'symbol/!' '-cvalidate'", \
+	"/usr/share/tex4ht/htlatex $$i 'html,word' 'symbol/!' '-cvalidate'"],
+        rc_entry = [ r'\converter latex      wordhtml   "%%"	"needaux"' ])
+    if htmlconv.find('htlatex') >= 0:
+      addToRC(r'''\copier    wordhtml       "python -tt $$s/scripts/ext_copy.py -e html,png,css $$i $$o"''')
     #
     checkProg('an OpenOffice.org -> LaTeX converter', ['w2l -clean $$i'],
         rc_entry = [ r'\converter sxw        latex      "%%"	""' ])
     #
-    checkProg('a LaTeX -> OpenOffice.org converter', ['oolatex $$i', 'oolatex.sh $$i'],
-        rc_entry = [ r'\converter latex      sxw        "%%"	"latex"' ])
+    checkProg('an OpenDocument -> LaTeX converter', ['w2l -clean $$i'],
+        rc_entry = [ r'\converter odt        latex      "%%"	""' ])
+    # On SuSE the scripts have a .sh suffix, and on debian they are in /usr/share/tex4ht/
+    # Both SuSE and debian have oolatex
+    checkProg('a LaTeX -> Open Document converter', ['oolatex $$i', 'oolatex.sh $$i', \
+        '/usr/share/tex4ht/oolatex $$i', \
+        'htlatex $$i \'xhtml,ooffice\' \'ooffice/! -cmozhtf\' \'-coo\' \'-cvalidate\''],
+        rc_entry = [ r'\converter latex      odt        "%%"	"needaux"' ])
     # On windows it is called latex2rt.exe
     checkProg('a LaTeX -> RTF converter', ['latex2rtf -p -S -o $$o $$i', 'latex2rt -p -S -o $$o $$i'],
         rc_entry = [ r'\converter latex      rtf        "%%"	"needaux"' ])
@@ -365,6 +391,15 @@ def checkConverterEntries():
     checkProg('a PS to PDF converter', ['ps2pdf13 $$i $$o'],
         rc_entry = [ r'\converter ps         pdf        "%%"	""' ])
     #
+    checkProg('a PS to TXT converter', ['pstotext $$i > $$o'],
+        rc_entry = [ r'\converter ps         text2      "%%"	""' ])
+    #
+    checkProg('a PS to TXT converter', ['ps2ascii $$i $$o'],
+        rc_entry = [ r'\converter ps         text3      "%%"	""' ])
+    #
+    checkProg('a DVI to TXT converter', ['catdvi $$i > $$o'],
+        rc_entry = [ r'\converter dvi        text4      "%%"	""' ])
+    #
     checkProg('a DVI to PS converter', ['dvips -o $$o $$i'],
         rc_entry = [ r'\converter dvi        ps         "%%"	""' ])
     #
@@ -404,9 +439,13 @@ def checkConverterEntries():
     checkProg('an EMF -> EPS converter', ['metafile2eps $$i $$o', 'wmf2eps -o $$o $$i'],
         rc_entry = [ r'\converter emf        eps        "%%"	""'])
     #
-    checkProg('a EPS -> PDF converter', ['epstopdf'],
+    checkProg('an EPS -> PDF converter', ['epstopdf'],
         rc_entry = [ r'\converter eps        pdf        "epstopdf --outfile=$$o $$i"	""', ''])
     #
+    # no agr -> pdf converter, since the pdf library used by gracebat is not
+    # free software and therefore not compiled in in many installations.
+    # Fortunately, this is not a big problem, because we will use epstopdf to
+    # convert from agr to pdf via eps without loss of quality.
     checkProg('a Grace -> Image converter', ['gracebat'],
         rc_entry = [
             r'''\converter agr        eps        "gracebat -hardcopy -printfile $$o -hdevice EPS $$i 2>/dev/null"	""
@@ -415,11 +454,8 @@ def checkConverterEntries():
 \converter agr        ppm        "gracebat -hardcopy -printfile $$o -hdevice PNM $$i 2>/dev/null"	""''',
             ''])
     #
-    checkProg('a LaTeX -> HTML converter', ['htlatex $$i', 'tth  -t -e2 -L$$b < $$i > $$o', \
-        'latex2html -no_subdir -split 0 -show_section_numbers $$i', 'hevea -s $$i'],
-        rc_entry = [ r'\converter latex      html       "%%"	"originaldir,needaux"' ])
     #
-    path, lilypond = checkProg('a LilyPond -> ESP/PDF/PNG converter', ['lilypond'])
+    path, lilypond = checkProg('a LilyPond -> EPS/PDF/PNG converter', ['lilypond'])
     if (lilypond != ''):
         version_string = cmdOutput("lilypond --version")
         match = re.match('GNU LilyPond (\S+)', version_string)
@@ -437,10 +473,13 @@ def checkConverterEntries():
         else:
             print '+  found LilyPond, but could not extract version number.'
     #
+    checkProg('a Noteedit -> LilyPond converter', ['noteedit --export-lilypond $$i'],
+        rc_entry = [ r'\converter noteedit   lilypond   "%%"	""', ''])
+    #
     # FIXME: no rc_entry? comment it out
     # checkProg('Image converter', ['convert $$i $$o'])
     #
-    # Entried that do not need checkProg
+    # Entries that do not need checkProg
     addToRC(r'''\converter lyxpreview ppm        "python -tt $$s/scripts/lyxpreview2bitmap.py"	""
 \converter date       dateout    "python -tt $$s/scripts/date.py %d-%m-%Y > $$o"	""
 \converter docbook    docbook-xml "cp $$i $$o"	"xml"
@@ -448,6 +487,13 @@ def checkConverterEntries():
 \converter fig        pdftex     "python -tt $$s/scripts/fig2pdftex.py $$i $$o"	""
 \converter fig        pstex      "python -tt $$s/scripts/fig2pstex.py $$i $$o"	""
 \converter lyx        lyx13x     "python -tt $$s/lyx2lyx/lyx2lyx -t 221 $$i > $$o"	""
+\converter lyx        lyx14x     "python -tt $$s/lyx2lyx/lyx2lyx -t 245 $$i > $$o"	""
+\converter lyx        clyx       "python -tt $$s/lyx2lyx/lyx2lyx -c big5 -t 245 $$i > $$o"	""
+\converter lyx        jlyx       "python -tt $$s/lyx2lyx/lyx2lyx -c euc_jp -t 245 $$i > $$o"	""
+\converter lyx        klyx       "python -tt $$s/lyx2lyx/lyx2lyx -c euc_kr -t 245 $$i > $$o"	""
+\converter clyx       lyx        "python -tt $$s/lyx2lyx/lyx2lyx -c big5 $$i > $$o"	""
+\converter jlyx       lyx        "python -tt $$s/lyx2lyx/lyx2lyx -c euc_jp $$i > $$o"	""
+\converter klyx       lyx        "python -tt $$s/lyx2lyx/lyx2lyx -c euc_kr $$i > $$o"	""
 ''')
 
 
@@ -491,9 +537,9 @@ def checkOtherEntries():
     ''' entries other than Format and Converter '''
     checkProg('a *roff formatter', ['groff', 'nroff'],
         rc_entry = [
-            r'\ascii_roff_command "groff -t -Tlatin1 $$FName"',
-            r'\ascii_roff_command "tbl $$FName | nroff"',
-            r'\ascii_roff_command ""' ])
+            r'\plaintext_roff_command "groff -t -Tlatin1 $$FName"',
+            r'\plaintext_roff_command "tbl $$FName | nroff"',
+            r'\plaintext_roff_command ""' ])
     checkProg('ChkTeX', ['chktex -n1 -n3 -n6 -n9 -n22 -n25 -n30 -n38'],
         rc_entry = [ r'\chktex_command "%%"' ])
     checkProg('a spellchecker', ['ispell'],
@@ -513,13 +559,14 @@ def checkOtherEntries():
     addToRC(r'''\copier    fig        "python -tt $$s/scripts/fig_copy.py $$i $$o"
 \copier    pstex      "python -tt $$s/scripts/tex_copy.py $$i $$o $$l"
 \copier    pdftex     "python -tt $$s/scripts/tex_copy.py $$i $$o $$l"
+\copier    program    "python -tt $$s/scripts/ext_copy.py $$i $$o"
 ''')
 
 
 def processLayoutFile(file, bool_docbook, bool_linuxdoc):
     ''' process layout file and get a line of result
         
-        Declear line are like this: (article.layout, scrbook.layout, svjog.layout)
+        Declare lines look like this: (article.layout, scrbook.layout, svjog.layout)
         
         \DeclareLaTeXClass{article}
         \DeclareLaTeXClass[scrbook]{book (koma-script)}
@@ -532,7 +579,7 @@ def processLayoutFile(file, bool_docbook, bool_linuxdoc):
         "svjog" "svjour" "article (Springer - svjour/jog)" "false"
     '''
     classname = file.split(os.sep)[-1].split('.')[0]
-    # return ('LaTeX', '[a,b]', 'a', ',b,c', 'article') for \DeclearLaTeXClass[a,b,c]{article}
+    # return ('LaTeX', '[a,b]', 'a', ',b,c', 'article') for \DeclareLaTeXClass[a,b,c]{article}
     p = re.compile(r'\Declare(LaTeX|DocBook|LinuxDoc)Class\s*(\[([^,]*)(,.*)*\])*\s*{(.*)}')
     for line in open(file).readlines():
         res = p.search(line)
@@ -542,17 +589,23 @@ def processLayoutFile(file, bool_docbook, bool_linuxdoc):
             if opt == None:
                 opt = classname
             return '"%s" "%s" "%s" "%s"\n' % (classname, opt, desc, avai)
-    print "Layout file without \DeclearXXClass line. "
+    print "Layout file without \DeclareXXClass line. "
     sys.exit(2)
 
     
 def checkLatexConfig(check_config, bool_docbook, bool_linuxdoc):
-    ''' Explore the LaTeX configuration '''
+    ''' Explore the LaTeX configuration 
+        Return None (will be passed to sys.exit()) for success.
+    '''
     print 'checking LaTeX configuration... ',
-    # First, remove the files that we want to re-create
-    removeFiles(['textclass.lst', 'packages.lst', 'chkconfig.sed'])
-    #
-    if not check_config:
+    # if --without-latex-config is forced, or if there is no previous 
+    # version of textclass.lst, re-generate a default file.
+    if not os.path.isfile('textclass.lst') or not check_config:
+        # remove the files only if we want to regenerate
+        removeFiles(['textclass.lst', 'packages.lst', 'chkconfig.sed'])
+        #
+        # Then, generate a default textclass.lst. In case configure.py
+        # fails, we still have something to start lyx.
         print ' default values'
         print '+checking list of textclasses... '
         tx = open('textclass.lst', 'w')
@@ -582,20 +635,23 @@ def checkLatexConfig(check_config, bool_docbook, bool_linuxdoc):
                 tx.write(processLayoutFile(file, bool_docbook, bool_linuxdoc))
         tx.close()
         print '\tdone'
+    if not check_config:
+        return None
+    # the following will generate textclass.lst.tmp, and packages.lst.tmp
     else:
         print '\tauto'
         removeFiles(['wrap_chkconfig.ltx', 'chkconfig.vars', \
             'chkconfig.classes', 'chklayouts.tex'])
         rmcopy = False
         if not os.path.isfile( 'chkconfig.ltx' ):
-            shutil.copy( os.path.join(srcdir, 'chkconfig.ltx'),  'chkconfig.ltx' )
+            shutil.copyfile( os.path.join(srcdir, 'chkconfig.ltx'), 'chkconfig.ltx' )
             rmcopy = True
         writeToFile('wrap_chkconfig.ltx', '%s\n%s\n\\input{chkconfig.ltx}\n' \
             % (linuxdoc_cmd, docbook_cmd) )
         # Construct the list of classes to test for.
         # build the list of available layout files and convert it to commands
         # for chkconfig.ltx
-        p1 = re.compile(r'\Declare(LaTeX|DocBook|LinuxDoc)Class')
+        p1 = re.compile(r'\Declare(LaTeX|DocBook)Class')
         testclasses = list()
         for file in glob.glob( os.path.join('layouts', '*.layout') ) + \
             glob.glob( os.path.join(srcdir, 'layouts', '*.layout' ) ) :
@@ -624,7 +680,8 @@ def checkLatexConfig(check_config, bool_docbook, bool_linuxdoc):
                 break;
             if re.match('^\+', line):
                 print line,
-        fout.close()
+        # if the command succeeds, None will be returned
+        ret = fout.close()
         #
         # currently, values in chhkconfig are only used to set
         # \font_encoding
@@ -632,14 +689,7 @@ def checkLatexConfig(check_config, bool_docbook, bool_linuxdoc):
         for line in open('chkconfig.vars').readlines():
             key, val = re.sub('-', '_', line).split('=')
             val = val.strip()
-            try:
-                values[key] = val.strip("'")
-            except TypeError:
-                # workaround for python 2.2.0 and 2.2.1
-                tmp = val.split("'")
-                while tmp and not tmp[0]: tmp = tmp[1:]
-                while tmp and not tmp[-1]: tmp = tmp[:-1]
-                values[key] = "'".join(tmp)
+            values[key] = val.strip("'")
         # chk_fontenc may not exist 
         try:
             addToRC(r'\font_encoding "%s"' % values["chk_fontenc"])
@@ -647,6 +697,13 @@ def checkLatexConfig(check_config, bool_docbook, bool_linuxdoc):
             pass
         if rmcopy:   # remove the copied file
             removeFiles( [ 'chkconfig.ltx' ] )
+        # if configure successed, move textclass.lst.tmp to textclass.lst
+        # and packages.lst.tmp to packages.lst
+        if os.path.isfile('textclass.lst.tmp') and len(open('textclass.lst.tmp').read()) > 0 \
+            and os.path.isfile('packages.lst.tmp') and len(open('packages.lst.tmp').read()) > 0:
+            shutil.move('textclass.lst.tmp', 'textclass.lst')
+            shutil.move('packages.lst.tmp', 'packages.lst')
+        return ret
 
 
 def createLaTeXConfig():
@@ -712,7 +769,7 @@ def checkTeXAllowSpaces():
             latex_out = cmdOutput(LATEX + r""" "\nonstopmode\input{\"a b\"}" """)
         else:
             latex_out = cmdOutput(LATEX + r""" '\nonstopmode\input{"a b"}' """)
-        if latex_out.find('working') != -1:
+        if 'working' in latex_out:
             print 'yes'
             tex_allows_spaces = 'true'
         else:
@@ -793,6 +850,11 @@ Options:
         addToRC(r'\tex_expects_windows_paths %s' % windows_style_tex_paths)
     checkOtherEntries()
     # --without-latex-config can disable lyx_check_config
-    checkLatexConfig( lyx_check_config and LATEX != '', bool_docbook, bool_linuxdoc)
+    ret = checkLatexConfig(lyx_check_config and LATEX != '', 
+        bool_docbook, bool_linuxdoc)
     createLaTeXConfig()
     removeTempFiles()
+    # The return error code can be 256. Because most systems expect an error code
+    # in the range 0-127, 256 can be interpretted as 'success'. Because we expect
+    # a None for success, 'ret is not None' is used to exit.
+    sys.exit(ret is not None)
