@@ -30,7 +30,7 @@ try:
     import lyx2lyx_version
     version_lyx2lyx = lyx2lyx_version.version
 except: # we are running from build directory so assume the last version
-    version_lyx2lyx = '1.4.5'
+    version_lyx2lyx = '1.5.0svn'
 
 default_debug_level = 2
 
@@ -128,6 +128,8 @@ def get_encoding(language, inputencoding, format, cjk_encoding):
         return lang[language][3]
     if inputencoding == "":
         return "latin1"
+    if inputencoding == "utf8x":
+        return "utf8"
     # python does not know the alias latin9
     if inputencoding == "latin9":
         return "iso-8859-15"
@@ -539,7 +541,7 @@ class LyX_Base:
                     k = k + 1
 
             # trim empty lines in the end.
-            while par[-1].strip() == '' and par:
+            while par and par[-1].strip() == '':
                 par.pop()
 
             toc_par.append(Paragraph(section, par))
